# Poisson statistics analysis
#
# Copyright (C) 2020 Boud Roukema
#
# This Makefile is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This Makefile is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this Makefile.  If not, see <http://www.gnu.org/licenses/>.

py_poisson_progs = $(pysrcdir)/find_range.py \
        $(pysrcdir)/count_jumps_mod.py \
        $(pysrcdir)/get_noise.py \
        $(pysrcdir)/theil_sen_mod.py \
        $(pysrcdir)/plot_countries_mod.py \
        $(pysrcdir)/subpoisson.py \
        $(pysrcdir)/compare_WHO_WP.py \
	$(pysrcdir)/read_counts_JHU.py

py_poisson_figs = $(outdir)/probability_vs_N_full.eps \
        $(outdir)/phi_N_full.eps \
        $(outdir)/psi_N_full.eps \
        $(outdir)/phi_N_a.eps \
        $(outdir)/phi_N_b.eps \
        $(outdir)/phi_N_c.eps

py_compare_inputdata_figs = $(outdir)/WHO_vs_WP.eps

py_poisson_tables_dat = $(data-publish-dir)/phi_N_full.dat \
        $(data-publish-dir)/phi_N_28days.dat \
        $(data-publish-dir)/phi_N_14days.dat \
        $(data-publish-dir)/phi_N_07days.dat \
        $(data-publish-dir)/phi_N_full_jhu.dat \
        $(data-publish-dir)/phi_N_28days_jhu.dat \
        $(data-publish-dir)/phi_N_14days_jhu.dat \
        $(data-publish-dir)/phi_N_07days_jhu.dat

py_compare_inputdata_tables = $(outdir)/WHO_vs_WP.dat

py_poisson_tables_tex = $(subst $(data-publish-dir)/phi_N,$(texbdir)/low_phi_table,\
        $(subst .dat,.tex,$(py_poisson_tables_dat)))



check-poisson: $(outdir)/done-check-poisson

$(outdir)/done-check-poisson: | $(outdir)
	cp -pv $(py_poisson_progs) $(installdir)/bin/ \
	  && cd $(outdir) \
	  && find_range.py \
	  && get_noise.py \
	  && touch $(outdir)/done-check-poisson

$(outdir):
	mkdir -p $(outdir)

poisson: $(outdir)/done-compare-datasets $(outdir)/done-poisson \
             $(mtexdir)/poisson.tex $(py_poisson_tables_tex)

$(py_poisson_tables_dat): $(outdir)/done-poisson

$(outdir)/done-poisson: $(inputdatasets) \
                        $(outdir)/done-check-poisson \
                        | $(outdir) $(data-publish-dir)
	cd $(outdir) \
	  && sed -e 's/Bonaire, Sint/Bonaire; Sint/' \
	     -e 's/territory, including/territory; including/' \
	     -e 's/, ,Other/,XX,Other/' \
	     $(indir)/$(WHOCOVID19_FILENAME) \
	     > $(outdir)/$(WHOCOVID19_FILENAME).cleaned \
	  && numthreads=$(strip $(n_cpus)) \
	  && if [ 0$${numthreads} -eq 0 ]; then \
	       numthreads=$$($(installdir)/bin/nproc --all | \
	       LANG=C awk '{n=$$1-2; if(n<1)n=1; if(n>20)n=20; print n}'); fi \
	  && printf "numthreads=$${numthreads}\n" \
	  && for INPUT in JHU WP; do \
	       if [ "x$${INPUT}" = "xWP" ]; then \
	           INFILE=$(indir)/$(WIKIPEDIA_CHARTS_FILENAME); \
	           suffix=""; \
	       else \
	           INFILE=$(indir)/$(JHU_FILENAME); \
	           suffix=_jhu; \
	       fi
	       INPUT_FILE=$${INFILE} \
	       N_THRESHOLD_START=$(N_threshold) \
	       N_THRESHOLD_STOP=$(N_threshold) \
	       MIN_DAYS=$(min_days) \
	       STOP_DAYS=$(stop_days) \
	       N_MEDIAN_MODEL=$(N_median_model) \
	       N_LOWEST_PSI=$(N_lowest_psi) \
	       N_LOWEST_PHI=$(N_lowest_phi) \
	       N_PLOT_LOWEST_PHI=$(N_plot_lowest_phi) \
	       N_PLOT_MEDIAN_PHI=$(N_plot_median_phi) \
	       N_PLOT_N_MIN_LOWEST_PHI=$(N_plot_N_min_lowest_phi) \
	       N_STDERR_CLUSTER=$(N_stderr_cluster) \
	       DELTA_LOG10_CLUSTER=$(delta_log10_cluster) \
	       DELTA_LOG10_REFINE=$(delta_log10_refine) \
	       N_SUBSEQ_LENGTH_A=$(N_subseq_length_a) \
	       N_SUBSEQ_LENGTH_B=$(N_subseq_length_b) \
	       N_SUBSEQ_LENGTH_C=$(N_subseq_length_c) \
	       FIXED_RNG_SEED=$(fixed_rng_seed) \
	       FIXED_RNG_SEED_VALUE=$(fixed_rng_seed_value) \
	       ENABLE_DEV_OVERRIDE=$(enable_dev_override) \
	       N_CPUS=$${numthreads} \
	       subpoisson.py \
	    && if [ "x$${INPUT}" = "xWP" ]; then
	           cp -pv $(py_poisson_figs) $(texbdir)/ \
	           && countries_daily_figs=$$(ls $(outdir)/countries_daily*.eps 2>/dev/null) \
	           && if [ "x$${countries_daily_figs}" != "x" ]; then \
	                 cp -pv $${countries_daily_figs} $(texbdir)/; fi; \
	       fi \
	    && $(call print-copyright, $(data-publish-dir)/phi_N_full$${suffix}.dat, new) \
	    && cat $(outdir)/phi_N_full.dat >> \
	         $(data-publish-dir)/phi_N_full$${suffix}.dat \
	    && $(call print-copyright, $(data-publish-dir)/phi_N_28days$${suffix}.dat, new) \
	    && cat $(outdir)/phi_N_a.dat >> \
	         $(data-publish-dir)/phi_N_28days$${suffix}.dat \
	    && $(call print-copyright, $(data-publish-dir)/phi_N_14days$${suffix}.dat, new) \
	    && cat $(outdir)/phi_N_b.dat >> \
	         $(data-publish-dir)/phi_N_14days$${suffix}.dat \
	    && $(call print-copyright, $(data-publish-dir)/phi_N_07days$${suffix}.dat, new) \
	    && cat $(outdir)/phi_N_c.dat >> \
	         $(data-publish-dir)/phi_N_07days$${suffix}.dat; \
	    done \
	  && touch $(outdir)/done-poisson

$(outdir)/done-compare-datasets: $(inputdatasets) | $(outdir) $(data-publish-dir)
	cd $(outdir) \
	  && N_THRESHOLD_START=$(N_threshold) \
	     INPUT_FILE_0=$(outdir)/$(WHOCOVID19_FILENAME).cleaned \
	     INPUT_FILE_1=$(indir)/$(WIKIPEDIA_CHARTS_FILENAME) \
	     compare_WHO_WP.py \
	  && cp -pv $(py_compare_inputdata_figs) $(texbdir)/ \
	  && $(call print-copyright, $(data-publish-dir)/WHO_vs_WP_jumps.dat, new) \
	  && cat $(py_compare_inputdata_tables) >> \
	       $(data-publish-dir)/WHO_vs_WP_jumps.dat \
	  && touch $(outdir)/done-compare-datasets



$(mtexdir)/poisson.tex: $(outdir)/done-poisson $(outdir)/done-compare-datasets \
                           $(py_poisson_tables_tex)
        # Produce a file with LaTeX macros for the analysis.
	printf "%% Automatically produced file.\n\n" \
	 && printf "%% Automatically produced file.\n\n" > $@ \
	 && for paramname in N_threshold min_days stop_days \
	      N_median_model N_lowest_phi N_lowest_psi N_stderr_cluster \
	      N_plot_lowest_phi N_plot_median_phi N_plot_N_min_lowest_phi \
	      N_subseq_length_a N_subseq_length_b N_subseq_length_c; do \
	      $(call create_tex_macro, $(pconfdir)/subpoisson.conf, $${paramname}, int, poiss); done \
	 && for paramname in N_countries_all N_neg N_counts_all N_countries_OK; do \
	      $(call create_tex_macro, $(outdir)/countries_out_full.dat, $${paramname}, int, poiss); done \
	 && for paramname in phi_N_full_slope phi_N_full_sig_slope phi_N_full_zeropoint phi_N_full_sig_zeropoint; do \
	      $(call create_tex_macro, $(outdir)/countries_out_full.dat, $${paramname}, float, poiss); done \
	 && for paramname in phi_N_a_slope phi_N_a_sig_slope phi_N_a_zeropoint phi_N_a_sig_zeropoint; do \
	      $(call create_tex_macro, $(outdir)/countries_out_a.dat, $${paramname}, float, poiss); done \
	 && for paramname in phi_N_b_slope phi_N_b_sig_slope phi_N_b_zeropoint phi_N_b_sig_zeropoint; do \
	      $(call create_tex_macro, $(outdir)/countries_out_b.dat, $${paramname}, float, poiss); done \
	 && for paramname in phi_N_c_slope phi_N_c_sig_slope phi_N_c_zeropoint phi_N_c_sig_zeropoint; do \
	      $(call create_tex_macro, $(outdir)/countries_out_c.dat, $${paramname}, float, poiss); done \
	 && for paramname in N_lowest_phi_countries N_lowest_psi_countries \
	      $$(grep "^[^#]*countries_daily_" $(outdir)/countries_out_full.dat | awk '{print $$1}'); do \
	      $(call create_tex_macro, $(outdir)/countries_out_full.dat, $${paramname}, str, poiss); done \
	 && for paramname in N_lowest_phi_below_half_countries N_lowest_phi_below_three_countries \
	      $$(grep "^[^#]*countries_daily_" $(outdir)/countries_out_a.dat | awk '{print $$1}'); do \
	      $(call create_tex_macro, $(outdir)/countries_out_a.dat, $${paramname}, str, poiss); done \
	 && for paramname in $$(grep "^[^#]*countries_daily_" $(outdir)/countries_out_b.dat | awk '{print $$1}'); do \
	      $(call create_tex_macro, $(outdir)/countries_out_b.dat, $${paramname}, str, poiss); done \
	 && for paramname in $$(grep "^[^#]*countries_daily_" $(outdir)/countries_out_c.dat | awk '{print $$1}'); do \
	      $(call create_tex_macro, $(outdir)/countries_out_c.dat, $${paramname}, str, poiss); done \
	 \
	 && for paramname in N_WHO_countries N_WPCCTF_countries N_common_countries; do \
	     $(call create_tex_macro, $(outdir)/WHO_vs_WP_countries.dat, $${paramname}, int, poiss); done \
	 && for country in BR US DZ BY RU IN FI PL; do \
	      for subseq in full a b c; do \
	         paramname=Phi$${country}Seq$${subseq}; \
	         phi_value=$$(grep "^$${country}" $(outdir)/phi_N_$${subseq}.dat | awk '{printf "%.2g",$$7}'); \
	         printf "\\\\newcommand{\\\\$${paramname}}{$${phi_value}}\n" >> $(mtexdir)/poisson.tex; \
	         paramname=Phi$${country}Seq$${subseq}SigLogTen; \
	         phi_value=$$(grep "^$${country}" $(outdir)/phi_N_$${subseq}.dat | awk '{printf "%.2g",$$8}'); \
	         printf "\\\\newcommand{\\\\$${paramname}}{$${phi_value}}\n" >> $(mtexdir)/poisson.tex; \
	         paramname=Phi$${country}Seq$${subseq}ThreeSigFig; \
	         phi_value=$$(grep "^$${country}" $(outdir)/phi_N_$${subseq}.dat | awk '{printf "%.3g",$$7}'); \
	         printf "\\\\newcommand{\\\\$${paramname}}{$${phi_value}}\n" >> $(mtexdir)/poisson.tex; \
	      done; \
	    done \
	 && printf "\nThe LaTeX macro file $(mtexdir)/poisson.tex was created.\n\n"

$(py_poisson_tables_tex): $(py_poisson_tables_dat) $(outdir)/done-poisson
	data_file=$(strip $(subst $(texbdir)/low_phi_table,$(data-publish-dir)/phi_N,$(subst .tex,.dat,$@)))
	printf "DEBUG poisson.mk; @ = $@  data_file=$${data_file}\n"
        ## phi
	printf '\\rule{0ex}{2.6ex}%%\n' > $@
	if (echo $@ |grep full); then \
	  grep -v "^ *#" $${data_file} | \
	  LANG=C awk '{printf "%2s %7d %.2f %.2f %.2f %.3f\n", $$1,$$2,$$5,$$6,$$7,int(1000*($$7/sqrt($$2)))/1000}' | \
	    LANG=C sort -g -k5,5|head -n$(N_lowest_phi) | \
	    sed -e 's/  */ \& /g' -e "s/\'/ \\\\\\\\/" >> $@; \
	else \
	  grep -v "^ *#" $${data_file} | \
	  LANG=C awk '{printf "%2s %7d %.1f %.2f %.2f %.2f %10s\n", $$1,$$2,$$4,$$5,$$6,$$7,$$9}' | \
	    LANG=C sort -g -k6,6|head -n$(N_lowest_phi) | \
	    sed -e 's/  */ \& /g' -e "s/\'/ \\\\\\\\/" >> $@; \
	fi
        ## psi
	if (echo $@ |grep full); then \
	  printf '\\hline\\rule{0ex}{2.6ex}%%\n' >> $@
	  grep -v "^ *#" $${data_file} | \
	  LANG=C awk '{printf "%2s %7d %.2f %.2f %.2f %.3f\n", $$1,$$2,$$5,$$6,$$7,int(1000*($$7/sqrt($$2)))/1000}' | \
	    LANG=C sort -g -k6,6|head -n$(N_lowest_psi) | \
	    sed -e 's/  */ \& /g' -e "s/\'/ \\\\\\\\/" >> $@; \
	else \
	  #printf '\\hline\\rule{0ex}{2.6ex}\n' >> $@
	  # grep -v "^ *#" $${data_file} | \
	  #LANG=C awk '{printf "%2s %7d  %.2f %.2f %.5f %10s\n", $$1,$$2,$$4,$$5,int(100000*($$5/sqrt($$2)))/100000,$$7}' $${data_file} | \
	  #  LANG=C sort -g -k5,5|head -n$(N_lowest_psi) | \
	  #  sed -e 's/  */ \& /g' -e "s/\'/ \\\\\\\\/" >> $@;
	  printf "Unjustified. Do not add to table.\n"
	fi
	printf "Have written out $@ .\n"

clean-poisson:
	rm -f $(outdir)/[a-z]* $(mtexdir)/poisson.tex $(texbdir)/low_phi_table.tex $(data-publish-dir)/*



.PHONY: check-poisson poisson clean-poisson
